# CONTENTS OF THIS FILE

 * Introduction
 * Installation
 * Requirements
 * Recommended modules
 * Configuration / Customization
 * Automatic attribution of resources
 * Cookies
 * Troubleshooting
 * Maintainers

## INTRODUCTION

This module implements the [Klaro! consent manager JS-Library](https://github.com/klaro-org/klaro-js) for Drupal and adds an interface to configure and customize klaro, manage apps and purposes, manage texts and their translations as well as automatically setting the required html-attributes to external resources for the JS-library to work. It also adds the ability to block unknown(No app configured) external resources by default.

### What is it good for?
The primary use case for this module is to:

- **Provide** a configurable consent/cookie manager for the site visitor, to
  give (or decline) consent to predefined "apps".
- **Satisfy** the european privacy laws and directives like [GDPR](https://eur-lex.europa.eu/eli/reg/2016/679/oj) or [ePrivacy Directive](https://eur-lex.europa.eu/eli/dir/2002/58/oj)
- **Manipulate** scripts and embeds (iframe, img, audio, video) added by
  editors or Drupal and/or its modules to adhere to the users' preference.

### Goals
- Privacy by default.
- Fully customizable, manageable and translatable consents (*apps*).
- Fully customizable, manageable and translatable *purposes*.
- Fully customizable and translatable texts of the consent manager.
- Automatically set all required html attributes and/or contextual blocking
  elements to external resources.
- Find all uncovered external resources.

## INSTALLATION

The installation of this module is like other Drupal modules.

1. Place the klaro Drupal module in the `modules` directory of your Drupal
   installation.
2. The [`klaro-org/klaro-js`](https://github.com/klaro-org/klaro-js/tags)
   javascript library will be installed as requirement to your `libraries` folder.
3. Enable the 'Klaro!' module in 'Extend': `/admin/modules`.
4. Set up user permissions: `/admin/people/permissions#module-klaro`, this
   module will add 2 permissions: `Administer Klaro!` to administrate klaro
   from in the backend. And `Use Klaro! UI` for clients/users to manage their
   consents, so make sure to add this one at least for guests/anonymous users.

### Using composer

We recommend to use composer for step 1 and 2.

For installing the Drupal module use:

`composer require drupal/klaro`

The package drupal/klaro_js is required by drupal/klaro and will install the
javascript library. (You may have to whitelist `drupal/klaro_js` in your
definition of `repo.packagist.org` in your project's composer.json.)

The original library is [klaro-org/klaro-js](https://github.com/klaro-org/klaro-js/).
The maintainers of drupal/klaro_js keep the package synchronized with the
original package.

The `composer.libraries.json` was removed in RC8.

If you want to keep on track for your Software Bill od Materials, you can add this composer entry to your composer.json:

```json
    "repositories": {
        [...],
        "klaro-org.klaro-js": {
            "type": "package",
            "package": {
                "name": "klaro-org/klaro-js",
                "type": "drupal-library",
                "version": "0.7.22",
                "dist": {
                    "url": "https://github.com/klaro-org/klaro-js/archive/refs/tags/v0.7.22.zip",
                    "type": "zip"
                }
            }
        }
    }
```

and then require the original package:

```php
composer require "klaro-org/klaro-js":"0.7.22"
```

## REQUIREMENTS

This module only requires the [`klaro-org/klaro-js`](https://github.com/klaro-org/klaro-js)
javascript library placed at `{web_dir}/libraries/klaro/dist`. No other Drupal modules are required.

## RECOMMENDED MODULES

* [Configuration Translation](https://www.drupal.org/docs/8/core/modules/config-translation) for multilingual sites.

## CONFIGURATION / CUSTOMIZATION

The module comes with some pre-defined *purposes*, such as "CMS" for
Drupal-related cookies or "External embeds". These are used to group
individual cookies, trackers etc. It also ships with some *app* settings,
e.g. for analytics provided by Matomo. You have to review and enable the apps
you need for your site.

### Apps for YouTube and Vimeo

Klaro offers 2 apps for external embeds: YouTube and Vimeo. Both apps are activated by default and take effect in the oembed field of Core Media Remote Video.

### App Matomo

For Matomo the module provides two different configurations. If you want to
track every visit and block only cookies, please use the app `matomo_cookies`.
You have to add an additional javascript line to the Matomo config on
`/admin/config/system/matomo`. Insert in "Advanced settings" in
"Code snippet (before)" the following command:
`_paq.push(['requireCookieConsent']);`. For further information see [here](https://matomo.org/faq/how-to/using-klaro-consent-manager-with-matomo/#klaro-open-source)
and [Issue #3346662](https://www.drupal.org/project/matomo/issues/3346662).

### Apps for Google Tag Manager / Analytics / Google Consent Mode v2

The module includes apps for Google Tag Manager and Google Analytics (GA4).
As GA4 is mostly used with the GTM, the GA4 app is obsolete in most cases.

The apps are designed for the Drupal modules [GoogleTag Manager](https://www.drupal.org/project/gtm) and
[Google Analytics 4](https://www.drupal.org/project/ga4_google_analytics).

Both apps can be activated and control the integration of Google code.

It is also possible to integrate the Google Tag Manager (GTM) with
**Google Consent Mode v2**, see [documentation](https://klaro.org/docs/tutorials/google_tag_manager).
See also [Issue #3484827](https://www.drupal.org/project/klaro/issues/3484827).

### Backend / UI

* Manage general settings&nbsp;`/admin/config/user-interface/klaro`.
* Manage apps&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;`/admin/config/user-interface/klaro/apps`.
* Manage purposes&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;`/admin/config/user-interface/klaro/purposes`.
* Manage texts&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;`/admin/config/user-interface/klaro/texts`.

### Styling

At `/admin/config/user-interface/klaro` settings->styling you can add additional
css classes that will be added to the consent-notice, the consent-modal and the
contextual-blocking element, so you can apply your own css to it.

In the field `Override Klaro css variables` you can make changes to positioning
and the used theme, for example you can enter `light, right, bottom` to use the
light-theme and position the cookie notice in the bottom right corner. More 
infos you find [here](https://github.com/klaro-org/klaro-js/blob/fb4e393d2cd8aeedc3e751d103dfbfd35ffae0f2/src/themes.js)

The setting "additional css classes" will be added to all klaro elements
(cookie-notice, consent-dialog, contextual blocking element), so you can
apply your own css to it.

For the toggle button shipped with this module, there is an override class
added `klaro_toggle_dialog_override` so you can overwrite the styles with
this css selector: `klaro_toggle_dialog.klaro_toggle_dialog_override`

### Open the Klaro! consent manager modal

* If the user saves a consent, the consent modal and/or cookie notice will
  disappear in the vanilla JS-Library until the cookies that save this
  information are deleted.
* This module ships a button to toggle the dialog, you can enable/disable it
  `/admin/config/user-interface/klaro` Settings->General->Show button to
  toggle the consent modal.
* If you like to provide own links or buttons to open the consent dialog at
  any time, you can just add a `rel`-attribute of any clickable element:
  `rel="open-consent-manager"`.
* If there is / are already any `rel`-value(s) just extend the value e.g.
  `rel="nofollow open-consent-manager noindex"`.

## AUTOMATIC ATTRIBUTION OF RESOURCES

The vanilla klaro library requires you to manually add html-attributes
`data-name="{app_name}` `data-src="{src}"` and `type="text/plain`
(for scripts or links) to your script, iframe, img, audio, video etc. tags.

This module tries to automate the process as far as possible by utilizing
js_alter, attachments_alter, preprocess_field and a kernelResponseListener
that will go through the final html and attribute all external resources.
It will also take care for external resources that are added via Drupal's
ajax-insert-commands (AfterCommand, AppendCommand, BeforeCommand,
HtmlCommand,PrependCommand, InsertCommand and ReplaceCommand) as well as on
the fly added external libraries with Drupal's add_js ajax command.

It does it by matching against the advanced configuration for each app. You
find those under `/admin/config/user-interface/klaro/apps/{app_name}`

In the field `sources` you can add paths as they appear in the src attribute
of script, iframe, img, video and audio tags, Enter one source per line,
partial matches are supported.

The preprocess_field handles at this time the "oembed" field formatter (from
media core) and the field type "iframe" from iframe module. If you need
further field type, please open a feature request and (optionally) open a
merge request (see below).

In the field `Embed wrapper classes` you can add css-classnames for which a
contextual blocking element will be wrapped. In example a twitter embed not
only contains a script tag (which will be blocked by adding the `sources`
field) but also some html, in this case a blockquote with the class
"twitter-tweet".

### Automatic blocking of unknown resources

Blocking unknown external resources is enabled by default. It will
automatically add an "unknown app" and let users decide to consent to
load them, by default a log notice will be added to recent log messages
if an external resource was processed, you can disable the logging
and also configure the label and description of the
unknown app or disable this feature completely.

Known bug: The HTML parser destroys special chars if TWIG Debug mode is enabled (see [Issue 3483397](https://www.drupal.org/project/klaro/issues/3483397)).

### What will be blocked automatically:
* script tags with src attribute
* iframes with field type "iframe"
* oembed with field formatter "oembed"

### What will be blocked automatically with "Process final HTML":
* script tags with src attribute
* img tags with src attribute (Contextual blocking wrap will be added)
* input type "image" tags with src attribute (Contextual blocking wrap will be
  added)
* link tags with href attribute (e.g fonts)
* audio tags with src attributes or source child nodes with src attribute
  (Contextual blocking wrap will be added)
* video tags with src attributes or source child nodes with src attribute
  (Contextual blocking wrap will be added)
* Elements that match the "Embed wrapper class" configuration of an app.
  (Contextual blocking wrap will be added)
* All above dynamically loaded with AJAX, including attached libraries.

### What will **not** be blocked automatically:
* Inline script tags with a script-body that will itself load external
  resources and are not added with page_attachments and a configured
  attachments-identifier in a klaro-app (You need to set the attributes
  manually to block it with klaro).
* Includes within css files.

### Logging of unknown resources

To find unknown external resources you can enable the option "Log unknown
resources" in tab "Automatic attribution" of the Library settings. After
activation, Drupal generates a log entry each time an external integration
is found in the HTML code of the page. It is recommended to activate this only
temporarily, as parsing the finished HTML document is time-consuming.

## COOKIES

Klaro saves the user-decisions as cookies(You can also configure to use the
browsers localstorage). However to let klaro also delete the app cookies,
i.e. when you revoke a consent, there are two places to provide information
about them.

1. `/admin/config/user-interface/klaro/apps/{klaro_app}`
   Inside the `Advanced` section you can provide specific cookie information
   about `name` (regex), `path` and `domain`
2. `/admin/config/user-interface/klaro`
   Inside the `Advanced` section you will find the `Matching cookie domains`
   textarea. Sometimes scripts may set cookies with dynamic cookie domains.
   You can add different domains here so that klaro will try to delete all
   cookies defined in your apps additionally with this cookie domains. So
   you don't need to add 30 cookie information inside your app just because
   a script adds several cookies just with 10 different cookie domains.

## TROUBLESHOOTING

- If the script- / resource **element attributes** won't appear, then there may
  be some other module or the theme that is preprocessing the tags and
  stripping out these attributes.
- Issue tracker: https://www.drupal.org/project/issues/klaro

## Use Klaro with custom preprocess_field.

If you want to implement Klaro for your own field types, you can
use the functions of the KlaroHelper class.
See `klaro_preprocess_field()` in `klaro.module`:

```php
  /** @var \Drupal\klaro\Utility\KlaroHelper $helper */
  $helper = \Drupal::service('klaro.helper');

  // Rewrite src from field type iframe.
  if ($variables["field_type"] == "iframe") {
    foreach ($variables['items'] as $i => $item) {
      $src = $item['content']['#src'];
      $app = $helper->matchKlaroApp($src);
      if ($app) {
        $attributes = $item['content']['#attributes'];
        $attributes = $helper->rewriteAttributes($attributes, $app->id());
        $variables['items'][$i]['content']['#attributes'] = $attributes;
      }
    }
  }
```

## Maintainers

  * Sascha Meißner ([sascha_meissner](https://www.drupal.org/u/sascha_meissner))
  * Jan Kellermann ([jan kellermann](https://www.drupal.org/u/jan-kellermann))

Thanks to

  * Christian Kipke ([ckidow](https://www.drupal.org/u/ckidow)) for first D8 Release.
  * Jürgen Haas ([jurgenhaas](https://www.drupal.org/u/jurgenhaas)) for supporting the V3 release.